<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: AdServer.php 3951 2005-12-12 11:33:16Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/Max.php';
require_once MAX_PATH . '/lib/core/ServiceLocator.php';
require_once MAX_PATH . '/lib/max/Maintenance/Statistics/Common.php';

/**
 * A class for summarising raw data into the various statistics that Max needs
 * for the AdServer module.
 *
 * @package    MaxMaintenance
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@m3.net>
 */
class Maintenance_Statistics_AdServer extends Maintenance_Statistics_Common
{

    /**
     * The constructor method.
     */
    function Maintenance_Statistics_AdServer()
    {
        $this->module = 'AdServer';
        parent::Maintenance_Statistics_Common();
    }

    /**
     * A method to summarise raw data into the intermediate statistics tables.
     */
    function summariseIntermediate()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if (($this->updateHourly) && (!$this->updateUsingOI)) {
            // Summarise the intermediate stats by the hour
            $start = &new Date();
            $start->copy($this->lastDateHourly);
            $start->addSeconds(1);
            while (Date::compare($start, $this->upToDateHourly) < 0) {
                // Calculate the end of the hour
                $end = &new Date();
                $end->copy($start);
                $end->addSeconds(3600 - 1); // Plus one hour
                // Summarise the requests
                $this->_summariseIntermediateRequests($start, $end);
                // Summarise the impressions
                $this->_summariseIntermediateImpressions($start, $end);
                // Summarise the clicks
                $this->_summariseIntermediateClicks($start, $end);
                if ($conf['modules']['Tracker']) {
                    // Summarise the connections
                    $this->_summariseIntermediateConnections($start, $end);
                }
                // Save the impressions/clicks/connections
                $this->_saveIntermediateSummaries($start, $end);
                // Go to the next hour
                $start->addSeconds(3600);
            }
        } elseif (($this->updateOI) && ($this->updateUsingOI)) {
            // Summarise the intermediate stats by the operation interval
            $start = &new Date();
            $start->copy($this->lastDateOI);
            $start->addSeconds(1);
            while (Date::compare($start, $this->upToDateOI) < 0) {
                // Calcuate the end of the operation interval
                $end = &new Date();
                $end->copy($start);
                $end->addSeconds(($conf['maintenance']['operationInterval'] * 60) - 1); // Plus one OI
                // Summarise the requests
                $this->_summariseIntermediateRequests($start, $end);
                // Summarise the impressions
                $this->_summariseIntermediateImpressions($start, $end);
                // Summarise the clicks
                $this->_summariseIntermediateClicks($start, $end);
                if ($conf['modules']['Tracker']) {
                    // Summarise the connections
                    $this->_summariseIntermediateConnections($start, $end);
                }
                // Save the impressions/clicks/connections
                $this->_saveIntermediateSummaries($start, $end);
                // Go to the next operation interval
                $start->addSeconds($conf['maintenance']['operationInterval'] * 60);
            }
        }
    }

    /**
     * A private method to summarise the ad requests, ready for insertion into
     * intermediate statistics tables.
     *
     * @access private
     * @param Date $start The start date of the data to summarise.
     * @param Date $end The end date of the data to summarise.
     */
    function _summariseIntermediateRequests($start, $end)
    {
        $startTime = time();
        $this->report .= "Summarising requests for the intermediate tables.\n";
        MAX::debug('Summarising requests for the intermediate tables', PEAR_LOG_DEBUG);
        $this->report .= 'Summarising requests between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                         $end->format('%Y-%m-%d %H:%M:%S') . ".\n";
        MAX::debug('Summarising requests between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                   $end->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        $rows = $this->dal->summariseRequests($start, $end);
        $runTime = time() - $startTime;
        $this->report .= "Summarised $rows request rows in $runTime seconds.\n";
        MAX::debug("Summarised $rows request rows in $runTime seconds", PEAR_LOG_DEBUG);
    }

    /**
     * A private method to summarise the ad impressions, ready for insertion into
     * intermediate statistics tables.
     *
     * @access private
     * @param Date $start The start date of the data to summarise.
     * @param Date $end The end date of the data to summarise.
     */
    function _summariseIntermediateImpressions($start, $end)
    {
        $startTime = time();
        $this->report .= "Summarising impressions for the intermediate tables.\n";
        MAX::debug('Summarising impressions for the intermediate tables', PEAR_LOG_DEBUG);
        $this->report .= 'Summarising impressions between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                         $end->format('%Y-%m-%d %H:%M:%S') . ".\n";
        MAX::debug('Summarising impressions between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                   $end->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);

        $rows = $this->dal->summariseImpressions($start, $end);
        $runTime = time() - $startTime;

        $this->report .= "Summarised $rows impression rows in $runTime seconds.\n";
        MAX::debug("Summarised $rows impression rows in $runTime seconds", PEAR_LOG_DEBUG);
    }

    /**
     * A private method to summarise the ad clicks, ready for insertion into
     * intermediate statistics tables.
     *
     * @access private
     * @param Date $start The start date of the data to summarise.
     * @param Date $end The end date of the data to summarise.
     */
    function _summariseIntermediateClicks($start, $end)
    {
        $startTime = time();
        $this->report .= "Summarising clicks for the intermediate tables.\n";
        MAX::debug('Summarising clicks for the intermediate tables', PEAR_LOG_DEBUG);
        $this->report .= 'Summarising clicks between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                         $end->format('%Y-%m-%d %H:%M:%S') . ".\n";
        MAX::debug('Summarising clicks between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                   $end->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        $rows = $this->dal->summariseClicks($start, $end);
        $runTime = time() - $startTime;
        $this->report .= "Summarised $rows click rows in $runTime seconds.\n";
        MAX::debug("Summarised $rows click rows in $runTime seconds", PEAR_LOG_DEBUG);
    }

    /**
     * A private method to summarise the ad connections, ready for insertion into
     * intermediate statistics tables.
     *
     * @access private
     * @param Date $start The start date of the data to summarise.
     * @param Date $end The end date of the data to summarise.
     */
    function _summariseIntermediateConnections($start, $end)
    {
        $startTime = time();
        $this->report .= "Summarising connections for the intermediate tables.\n";
        MAX::debug('Summarising connections for the intermediate tables', PEAR_LOG_DEBUG);
        $this->report .= 'Summarising connections between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                         $end->format('%Y-%m-%d %H:%M:%S') . ".\n";
        MAX::debug('Summarising connections between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                   $end->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        $rows = $this->dal->summariseConnections($start, $end);
        $runTime = time() - $startTime;
        $this->report .= "Summarised $rows connection rows in $runTime seconds.\n";
        MAX::debug("Summarised $rows connection rows in $runTime seconds", PEAR_LOG_DEBUG);
    }

    /**
     * A private method to save the summarised request/impressions/clicks/connections
     * into the intermediate tables.
     *
     * @access private
     * @param Date $start The start date of the data to save.
     * @param Date $end The end date of the data to save.
     */
    function _saveIntermediateSummaries($start, $end)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if ($conf['modules']['Tracker']) {
            $this->report .= "Saving request, impression, click and connection data into the intermediate tables.\n";
            MAX::debug('Saving request, impression, click and connection data into the intermediate tables', PEAR_LOG_DEBUG);
        } else {
            $this->report .= "Saving request, impression, click data into the intermediate tables.\n";
            MAX::debug('Saving request, impression, click data into the intermediate tables', PEAR_LOG_DEBUG);
        }
        $this->dal->saveIntermediate($start, $end);
    }

    /**
     * A method to summarise intermediate statistics into summary (final) statistics
     * tables, where required.
     */
    function summariseFinal()
    {
        if ($this->updateOI) {
            // Update the zone impression history table
            $this->report .= 'Updating the zone impression history table for data after ' .
                             $this->lastDateOI->format('%Y-%m-%d %H:%M:%S') . ".\n";
            MAX::debug('Updating the zone impression history tables for data after ' .
                       $this->lastDateOI->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
            $start = &new Date();
            $start->copy($this->lastDateOI);
            $start->addSeconds(1);
            $this->dal->saveHistory($start, $this->upToDateOI);
        }
        if ($this->updateHourly) {
            // Update the hourly summary table
            $this->report .= 'Updating the hourly summary table for data after ' .
                             $this->lastDateHourly->format('%Y-%m-%d %H:%M:%S') . ".\n";
            MAX::debug('Updating the hourly summary table for data after ' .
                       $this->lastDateHourly->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
            $start = &new Date();
            $start->copy($this->lastDateHourly);
            $start->addSeconds(1);
            $this->dal->saveSummary($start, $this->upToDateHourly);
        }
    }

    /**
     * A method to manage the activation/deactivation of campaigns.
     */
    function manageCampaigns()
    {
        if ($this->updateOI) {
            $this->report .= "Activating/Deactivating campaigns.\n";
            $message .= 'Activating/Deactivating campaigns.';
            MAX::debug($message);
            $oServiceLocator = &ServiceLocator::instance();
            $oDate = &$oServiceLocator->get('now');
            $this->report .= $this->dal->manageCampaigns($oDate);
        }
    }

   /**
    * Method to override parent logCompletion class and pass in
    * run type for use when logging data to DB.
    */
    function logCompletion($now = null) {
        parent::logCompletion($now, 'adserver_run_type');
    }

}

?>
