<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: PriorityEntities.mtp.test.php 3944 2005-12-09 11:22:17Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/max/Maintenance/Priority/Entities.php';

/**
 * A class for testing the Maintenance_Priority_Constraint class.
 *
 * @package    MaxMaintenance
 * @subpackage TestSuite
 * @author     James Floyd<james@m3.net>
 */
class Maintenance_TestOfPriorityEntities extends UnitTestCase
{

   /**
    * The constructor method.
    */
    function  Maintenance_TestOfPriorityEntities()
    {
        $this->UnitTestCase();
        Mock::generate('MAX_Dal_Maintenance');
        Mock::generatePartial('Placement', 'PartialMockPlacement',
                              array('_getDal', 'getAdverts'));
        Mock::generatePartial('Advert', 'PartialMockAdvert',
                              array('_getDal'));
    }

   /*
    *************************
    * PLACEMENT CLASS TESTS *
    *************************
    */

    /**
     * Test 1 - Test object creation
     */
    function testPlacement()
    {
        // Mock the MAX_DB class used in the constructor method
        $oDal = &new MockMAX_Dal_Maintenance($this);
        // Partially mock the Placement class
        $oPlacement = &new PartialMockPlacement($this);
        $oPlacement->setReturnReference('_getDal', $oDal);
        $aArgs = array(
            'campaignid'        => 1,
            'expire'            => '2005-01-31',
            'views'             => 1000000,
            'clicks'            => 100000,
            'conversions'       => 1000,
            'target_impression' => 2,
            'target_click'      => 3,
            'target_conversion' => 4,
            'priority'          => 5
        );
        $oPlacement->Placement($aArgs);
        // Test the object properties
        foreach($aArgs as $key => $value) {
            if ($key == 'campaignid') {
                $this->assertEqual($oPlacement->id, $value, 'Looking at Placement object property ' . $key);
            } elseif ($key == 'views') {
                $this->assertEqual($oPlacement->targetImpressions, $value, 'Looking at Placement object property ' . $key);
            } elseif ($key == 'clicks') {
                $this->assertEqual($oPlacement->targetClicks, $value, 'Looking at Placement object property ' . $key);
            } elseif ($key == 'conversions') {
                $this->assertEqual($oPlacement->targetConversions, $value, 'Looking at Placement object property ' . $key);
            } elseif ($key == 'target_impression') {
                $this->assertEqual($oPlacement->dailyTargetImpressions, $value, 'Looking at Placement object property ' . $key);
            } elseif ($key == 'target_click') {
                $this->assertEqual($oPlacement->dailyTargetClicks, $value, 'Looking at Placement object property ' . $key);
            } elseif ($key == 'target_conversion') {
                $this->assertEqual($oPlacement->dailyTargetConversions, $value, 'Looking at Placement object property ' . $key);
            } else {
                $this->assertEqual($oPlacement->{$key}, $value, 'Looking at Placement object property ' . $key);
            }
        }
    }

    /**
     * Test 1 -  Test methods ability to return correct result from
     *           known value of
     */
    function testGetAdverts()
    {
        // Mock the MAX_DB class used in the constructor method
        $oDal = &new MockMAX_Dal_Maintenance($this);
        // Define array of adverts to be returned from mock DA layer
        $aAdvertVals = array(
            array('ad_id' => 1, 'type' => 'sql', 'weight' => 2, 'active' => 1,),
            array('ad_id' => 2, 'type' => 'gif', 'weight' => 1, 'active' => 1,),
            array('ad_id' => 3, 'type' => 'sql', 'weight' => 2, 'active' => 1,),
            array('ad_id' => 4, 'type' => 'gif', 'weight' => 3, 'active' => 1,),
        );
        // Partially mock the Campaign class
        $oPlacement = &new PartialMockPlacement($this);
        $oPlacement->setReturnReference('_getDal', $oDal);
        $oPlacement->setReturnValue('getAdverts', $aAdvertVals);
        $oPlacement->Placement();
        // Run test function
        $aAdverts = $oPlacement->getAdverts();
        // Count advert members of Campaign
        $this->assertEqual(4, count($aAdverts));
        // Check keys and values have been set
        foreach($aAdverts as $key => $oAdvert) {
            foreach($aAdvertVals as $propKey => $value) {
                $this->assertEqual($value[$propKey], $oAdvert->{$propKey});
            }
        }
    }

    /**
     * @TODO Test not implemented.
     */
    function testSetSummaryStatistics() {}

    /*
     **********************
     * ADVERT CLASS TESTS *
     **********************
     */

    /**
     * @TODO Test not implemented.
     */
    function testAdvert() {}

    /**
     * @TODO Test not implemented.
     */
    function testGetConstraints() {}

    function testSetRequiredImpressions()
    {
        // Mock the MAX_DB class used in the constructor method
        $oDal = &new MockMAX_Dal_Maintenance($this);
        // Partially mock the Advert class
        $oAdvert = &new PartialMockAdvert($this);
        $oAdvert->setReturnReference('_getDal', $oDal);
        $oAdvert->Advert();
        // A few simple setter tests
        $oAdvert->setRequiredImpressions(350);
        $this->assertEqual($oAdvert->requiredImpressions, 350);
        $oAdvert->setRequiredImpressions(0);
        $this->assertEqual($oAdvert->requiredImpressions, 0);
        $oAdvert->setRequiredImpressions(10);
        $this->assertEqual($oAdvert->requiredImpressions, 10);
    }

    /**
     * @TODO Test not implemented.
     */
    function testAllocateImpressions() {}

    /*
     **********************
     * ZONE CLASS TESTS *
     **********************
     */

    /**
     * @TODO Test not implemented.
     */
    function testZone() {}

    /**
     * @TODO Test not implemented.
     */
    function testAddAdvert() {}

    /**
     * @TODO Test not implemented.
     */
    function testContainsFilteredAds() {}

    /**
     * @TODO Test not implemented.
     */
    function testSetIntervalIdImpressionAverage() {}

    /**
     * @TODO Test not implemented.
     */
    function testSetIntervalIdImpressionForecast() {}

    /**
     * @TODO Test not implemented.
     */
    function testSetIntervalIdImpressionActual() {}

}

?>
