<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: campaign.plugin.php 3538 2005-08-19 14:42:58Z radek $
*/

/**
 * @package    MaxPlugins
 * @subpackage Reports
 * @author     Misza Pawlowski <michal@m3.net>
 * @author     Radek Maciaszek <radek@m3.net>
 */

require_once MAX_PATH . '/plugins/reports/Reports.php';
require_once MAX_PATH . '/plugins/reports/lib.php';

class Plugins_Reports_Publisher_Campaign extends Plugins_Reports {
    // Public info function
    function info()
    {	
    	include_once MAX_PATH . '/lib/max/Plugins/PluginTranslation.php';
        Max_PluginTranslation::init($this->module, $this->package);
        
        $plugininfo = array (
    		"plugin-name"			=> Max_PluginTranslation::translate('Campaign Analysis', $this->module, $this->package),
    		"plugin-description"	=> Max_PluginTranslation::translate('This table is an summary of the campaigns on the site.', $this->module, $this->package),
    		'plugin-category'     => 'publisher',
    		'plugin-category-name'	=> Max_PluginTranslation::translate('Publisher Reports', $this->module, $this->package),
    		"plugin-author"		=> "Michal Pawlowski",
    		'plugin-category'   => 'publisher',
    		"plugin-export"		=> "csv",
    		"plugin-authorize"	=> phpAds_Admin+phpAds_Agency+phpAds_Affiliate,
    		"plugin-import"		=> array (
    			"campaignid"	=> array (
    				"title"		=> $GLOBALS['strAffiliate'],
    				"type"		=> "affiliateid-dropdown" ),
    			"startdate"		=> array (
                    "title"		=> "Start Date",
                    "type"		=> "edit",
                    "size"		=> 10,
                    "default"	=> date("Y/m/d",mktime (0,0,0,date("m")-1,1,  date("Y")))
                ),
                "enddate"		=> array (
                    "title"		=> "End Date",
                    "type"		=> "edit",
                    "size"		=> 10,
                    "default"	=> date("Y/m/d",mktime (0,0,0,date("m"),1,  date("Y")))
                ),
    		 )
    	);
    	
    	return ($plugininfo);
    }
    
    /*-------------------------------------------------------*/
    /* Private plugin function                               */
    /*-------------------------------------------------------*/
    
    function execute($affiliateid, $startdate, $enddate)
    {
        require_once 'Spreadsheet/Excel/Writer.php';
        
    	global $date_format;
    	global $phpAds_ThousandsSeperator,$phpAds_DecimalPoint;
    	
    	$conf = & $GLOBALS['_MAX']['CONF'];
    	
    	$reportName = $GLOBALS['strPublisherCampaignAnalysisReport'];
    	
    	$startDate = date(str_replace('%','',$date_format), strtotime($startdate));
    	$endDate   = date(str_replace('%','',$date_format), strtotime($enddate));
    	$dbStart = date('Y-m-d', strtotime($startdate));
    	$dbEnd = date('Y-m-d', strtotime($enddate));
    	// creating report object
        $workbook = new Spreadsheet_Excel_Writer();
        
        $worksheet = generateXLSHeader($workbook, $reportName, $startDate, $endDate,
                                       $affiliateid, $GLOBALS['strPublisherCampaignAnalysisDescription']);
        // where to start printing structural data
        $rowStart = 7;
        $columnStart = 0;
        
        // formatting - begin
        $formatInteger =& $workbook->addFormat();
        $formatInteger->setNumFormat($GLOBALS['excell_integer_formatting']);
        
        $formatDecimal =& $workbook->addFormat();
        $formatDecimal->setNumFormat($GLOBALS['excell_decimal_formatting']);
        
        $columnHeader =& $workbook->addFormat();
        $columnHeader->setAlign('center');
        $columnHeader->setBold();
            
        $formatRight =& $workbook->addFormat();
        $formatRight->setAlign('right');
        // formating - end
        
        $query = "SELECT
                       c.campaignname
                      ,coalesce(sum(ds.requests),0)        as requests
                      ,coalesce(sum(ds.impressions),0)     as impressions
                      ,coalesce(sum(ds.clicks),0)          as clicks
                      ,coalesce(sum(ds.conversions),0)     as conversions
                  FROM ".$conf['table']['prefix'].$conf['table']['zones']." z
                      ,".$conf['table']['prefix'].$conf['table']['data_summary_ad_hourly']." ds
                      ,".$conf['table']['prefix'].$conf['table']['banners']." b
                      ,".$conf['table']['prefix'].$conf['table']['campaigns']." c
                 WHERE z.zoneid = ds.zone_id
                   AND ds.ad_id = b.bannerid
                   AND b.campaignid = c.campaignid
                   AND z.affiliateid = ".$affiliateid."
                   AND ds.day >= '$dbStart'
                   AND ds.day <= '$dbEnd'
                 GROUP BY c.campaignname, c.priority
                 ORDER BY day ASC";
        
    	$res = phpAds_dbQuery($query) or phpAds_sqlDie();
    	
    	if (phpAds_dbNumRows($res)) {
        	while ($row = phpAds_dbFetchArray($res)) {
        	    $data[] = $row;
        	}
        	
            $numrows = (is_array($data)) ? count($data): 0;
            
            // adding "total" rows
        	addTotals($data);
        	
        	$currentRow = $rowStart;
        	  
            $worksheet->write($currentRow,$columnStart,
                              $GLOBALS['strCampaign'],$columnHeader);
            $worksheet->write($currentRow,$columnStart + 1,
                              $GLOBALS['strRequests'],$columnHeader);
            $worksheet->write($currentRow,$columnStart + 2,
                              $GLOBALS['strImpressions'],$columnHeader);
            $worksheet->write($currentRow,$columnStart + 3,
                              $GLOBALS['strClicks'],$columnHeader);
            $worksheet->write($currentRow,$columnStart + 4,
                              $GLOBALS['strCTRShort'],$columnHeader);
            $worksheet->write($currentRow,$columnStart + 5,
                              $GLOBALS['strConversions'],$columnHeader);
            $worksheet->write($currentRow,$columnStart + 6,
                              $GLOBALS['strCNVRShort'], $columnHeader);
            
            $currentRow++;
        	foreach ($data as $rowId => $row) {
        	    $worksheet->write($currentRow, $columnStart, $row['campaignname']);
                $worksheet->writeNumber($currentRow, $columnStart + 1,
                                        $row['requests'],$formatInteger);
        	    $worksheet->writeNumber($currentRow, $columnStart + 2,
        	                            $row['impressions'],$formatInteger);
        	    $worksheet->writeNumber($currentRow, $columnStart + 3,
        	                            $row['clicks'], $formatInteger);
        	    if ($row['impressions'] != 0) {
        	       $worksheet->writeNumber($currentRow, $columnStart + 4, 
        	                               ($row['clicks'] / $row['impressions']) * 100, $formatDecimal);
        	    } else {
        	       $worksheet->write($currentRow, $columnStart + 4,"N/A");
        	    }
        	    $worksheet->writeNumber($currentRow, $columnStart + 5,
        	                            $row['conversions'], $formatInteger);
        	    if ($row['clicks'] != 0) {
        	       $worksheet->writeNumber($currentRow, $columnStart + 6,
        	                               ($row['conversions'] / $row['clicks']) * 100, $formatDecimal);
        	    } else {
        	       $worksheet->write($currentRow, $columnStart + 6,"N/A");
        	    }
        	    $currentRow++;
        	}
    	} else { // there is no data
    	    $worksheet->write($rowStart, $columnStart,
    	                      $GLOBALS['strNoData'], $columnHeader);
    	}
    	   
    	$workbook->close();
    }
}

?>